
#define W_RIGHT  3
#define W_BOTTOM 4
#define WIN_WANT_ALL
#define WIN_WANT_SBS
#define WIN_WANT_EN
#define WIN_WANT_LBS
#define WIN_WANT_LB
#define WIN_WANT_CB
#define WIN_WANT_MF
#define WIN_WANT_IDM
#define WIN_WANT_HELP
#define WIN_WANT_DRAWTEXT
#define WIN_WANT_RESOURCE
#define WIN_WANT_CLIPBOARD
#define WIN_WANT_SYSTEM_METRICS
#define WIN_WANT_GETDEVCAPS
#include "windows.ch"
#include "inkey.ch"
#include "accel.ch"
#include "textmetr.ch"
#include "dialog.ch"
#include "font.ch"
#include "demo.ch"

STATIC hMainWnd
STATIC HBALL1,HBALL2,HBALL3,HBALL4
STATIC cLastMsg
STATIC lCanProcess := .F.
STATIC nTH

// LoadBalls()
// Loads the 4 cursors used to make the spinning beach ball cursor.
//
FUNCTION LoadBalls
       LOCAL hInst := _GetInstance()

       HBALL1   := LoadCursor(hInst,"BALL1"+CHR(0))
       HBALL2   := LoadCursor(hInst,"BALL2"+CHR(0))
       HBALL3   := LoadCursor(hInst,"BALL3"+CHR(0))
       HBALL4   := LoadCursor(hInst,"BALL4"+CHR(0))
RETURN(NIL)

// SetMainWnd(hWnd)
// Sets the var hMainWnd so other routines can use it.
//
FUNCTION SetMainWnd(hWnd)

       hMainWnd := hWnd
       nTH      := TextHeight()
RETURN(NIL)

// GetMainWnd()
// Used to supply main window handle to other modules.
//
FUNCTION GetMainWnd
RETURN(hMainWnd)

// Displays a message on the message/status bar of a window.
//
FUNCTION Message(hWnd,cText)
       LOCAL nLeft, nTop, nRight, nBottom, aCRect, hDC
       LOCAL nOldBkClor

       nTH     := TextHeight()
       aCRect  := GetClientRect(hWnd)
       nLeft   := 0
       nTop    := aCRect[4]-(nTH+10)
       nRight  := aCRect[3]
       nBottom := nTop+(nTH+10)

       hDC        := GetDC(hWnd)
       nOldBkClor := SetBkColor(hDC,RGB(192,192,192))
       FillRect(hDC,nLeft+20,nTop+5,nRight-180,nBottom-5,GetStockObject(LTGRAY_BRUSH))
       DrawText(hDC,cText,;
               {nLeft+20,nTop+5,nRight-180,nBottom-5},DT_LEFT+DT_SINGLELINE)
       SetBkColor(hDC,nOldBkClor)
       ReleaseDC(hWnd,hDC)
       cLastMsg := cText
RETURN(NIL)

// Draws the status/message bar.
//
FUNCTION MsgBar(hWnd)
       LOCAL nLeft, nTop, nRight, nBottom, aCRect, hOldPen
       LOCAL hDC, hColor, hBlackPen, hGreyPen, hWhitePen, hBkClor

       hDc       := GetDC(hWnd)
       aCRect    := GetClientRect(hWnd)
       nLeft     := 0
       nTop      := aCRect[4]-(nTH+10)
       nRight    := aCRect[3]
       nBottom   := nTop+(nTh+10)
       hBkClor   := SetBkColor(hDC,RGB(192,192,192))
       hBlackPen := CreatePen(PS_SOLID,1,RGB(0,0,0))
       hGreyPen  := CreatePen(PS_SOLID,1,RGB(128,128,128))
       hWhitePen := CreatePen(PS_SOLID,1,RGB(255,255,255))

       hOldPen   := SelectObject(hDC,hBlackPen)
       Rectangle(hDC,nLeft,nTop,nRight,nBottom)
       hColor := CreateSolidBrush(RGB(192,192,192))
       FillRect(hDC,nLeft,nTop+1,nRight,nBottom,hColor)

       SelectObject(hDC,hGreyPen)
       MoveTo(hDC,nLeft+10,nBottom-4)
       LineTo(hDC,nLeft+10,nTop+4)
       LineTo(hDC,nRight-10,nTop+4)

       SelectObject(hDC,hWhitePen)
       LineTo(hDC,nRight-10,nBottom-4)
       LineTo(hDC,nLeft+10,nBottom-4)

       IF hWnd == hMainWnd
          CWinBox(hWnd, 08,nTop+3,nRight-168,nBottom-3,1,2,1,0,4)
          CWinBox(hWnd,nRight-168,nTop+3,nRight-86,nBottom-3,1,2,1,0,6)
          CWinBox(hWnd,nRight-88,nTop+3,nRight-6,nBottom-3,1,2,1,0,6)
          DrawText(hDC,DTOC(DATE()),{nRight-168,nTop+5,nRight-86,nBottom-5},DT_CENTER+DT_SINGLELINE)
          DrawText(hDC,TIME(),{nRight-88,nTop+5,nRight-6,nBottom-5},DT_CENTER+DT_SINGLELINE)
          SetBkColor(hDC,hBkClor)
       ENDIF
       SelectObject(hDC,hOldPen)
       ReleaseDC(hWnd,hDC)
       DeleteObject(hColor)
       DeleteObject(hBlackPen)
       DeleteObject(hGreyPen)
       DeleteObject(hWhitePen)
RETURN(NIL)

// Working()
// Displays a message on the message/status bar.
// Sets the cursor to the spinning balls.
//
FUNCTION Working(lOnOff,cText,cMsg)
       STATIC hOldCur, cOldMsg

       IF lOnOff == .T.
          IF cMsg != NIL
             cOldMsg := cMsg
          ENDIF
          hOldCur := SetCursor(HBALL1)
          IF cText != NIL
             Message(hMainWnd,cText)
          ENDIF
       ELSE
          SetCursor( LoadCursor(,IDC_ARROW ) )
          IF cOldMsg != NIL
             Message(hMainWnd,cOldMsg)
          ELSE
             SendMessage(hMainWnd, WM_PAINT,0)
          ENDIF
       ENDIF
RETURN(NIL)

// SpinBall(nDelay)
// Makes the spinning beach ball cursor spin.  If it is too fast
// you can specify a delay in 1/100 seconds.
//
FUNCTION SpinBall(nDelay)
       STATIC nBall := 1

       IF nDelay != NIL
          Delay(nDelay)
       ENDIF
       DO CASE
          CASE nBall == 1
               SetCursor(HBALL2)
               nBall := 2
          CASE nBall == 2
               SetCursor(HBALL3)
               nBall := 3
          CASE nBall == 3
               SetCursor(HBALL4)
               nBall := 4
          CASE nBall == 4
               SetCursor(HBALL1)
               nBall := 1
       ENDCASE
RETURN(.T.)

// Delay(nDelay)
// Crude delay function.
//
STATIC FUNCTION Delay(nDelay)

       DO WHILE (SECONDS()+(nDelay/100)) < SECONDS()
       ENDDO
RETURN(NIL)

// UpDtTime()
// Updates the time displayed on the status bar.  Called from within
// Dbrowse() and the main program module.
//
FUNCTION UpDtTime
       LOCAL hDC        := GetDC(hMainWnd)
       LOCAL nOldBkClor := SetBkColor(hDC,RGB(192,192,192))
       LOCAL aCRect     := GetClientRect(hMainWnd)
       LOCAL nLeft      := 0
       LOCAL nTop       := aCRect[4]-(nTH+10)
       LOCAL nRight     := aCRect[3]
       LOCAL nBottom    := nTop+(nTH+10)

       DrawText(hDC,TIME(),{nRight-88,nTop+6,nRight-6,nBottom-5},DT_CENTER+DT_SINGLELINE)
       SetBkColor(hDC,nOldBkClor)
       ReleaseDC(hMainWnd,hDC)
RETURN(NIL)

// LoadListBox(hWnd,bFields)
// Loads a listbox with values from a database.
// Make sure you have selected the database first!
//
FUNCTION LoadListBox(hWnd,bFields)
       LOCAL cFirstStr

       GO TOP
       cFirstStr := EVAL(bFields)
       DO WHILE !EOF()
          SendMessage(hWnd, LB_ADDSTRING, 0, EVAL(bFields))
          SKIP
       ENDDO
       SendMessage(hWnd, LB_SELECTSTRING, -1, cFirstStr)
RETURN(NIL)

// LBGetSelection(hLB)
// Returns the selected item in a list box.
//
FUNCTION LBGetSelection(hLB)
       LOCAL nCtr, cText

       nCtr  := SendMessage(hLB, LB_GETCURSEL, 0, 0)
       cText := SPACE(SendMessage(hLB, LB_GETTEXTLEN, nCtr, 0))
       SendMessage(hLB, LB_GETTEXT, nCtr, @cText)
RETURN(cText)

// Compile(cStr)
// Takes a string and returns a codeblock - runtime compiler
// Might as well make use of it since the Clipper compiler
// is always linked into your Clipper apps!
//
FUNCTION Compile(cStr)
RETURN(&("{||"+cStr+"}"))

// ErrorMsg(hWnd,cMarr,cType)
// General purpose error message handler
// Uses MessageBox() so if the user decides to move it the window
// underneath will NOT be automatically redrawn - You could always
// use a dialog box but the size would be fixed...
//
FUNCTION ErrorMsg(hWnd,cMarr,cType)
       LOCAL nResponse

       DO CASE
          CASE cType = 'W'
               MessageBeep(MB_ICONEXCLAMATION)
               nResponse := MessageBox(hWnd,cMarr,"Caution!",;
                            MB_OKCANCEL+MB_ICONEXCLAMATION)
          CASE cType = 'E'
               MessageBeep(MB_ICONHAND)
               nResponse := MessageBox(hWnd,cMarr,"Problem!",;
                            MB_OK+MB_ICONHAND)
          OTHERWISE
               MessageBeep(MB_OK)
               nResponse := MessageBox(hWnd,cMarr,"Please!",;
                            MB_OKCANCEL+MB_ICONASTERISK)
       ENDCASE
RETURN(nResponse)

// NegToPos(nValue)
// Converts a negative numeric value to a positive one
// Maybe not the most elegant way but it works!
//
FUNCTION NegToPos(nValue)
RETURN(VAL(STRTRAN(STR(nValue),'-')))

// PosToNeg(nValue)
// Converts a positive numeric value to a negative one
// Maybe not the best way to do it but it works!
//
FUNCTION PosToNeg(nValue)
RETURN(VAL('-'+LTRIM(STR(nValue))))

// CenterWindow() - Courtesy of Gerald Barber
// Centers a window in it's parent windows client area
// Written by Gerald Barber - thanks Gerald!
//
FUNCTION CenterWindow(hWnd)
       LOCAL hWndParent
       LOCAL aChild_[4]
       LOCAL iCWidth
       LOCAL iCHeight
       LOCAL aParent_[4]
       LOCAL aPoint_[2]

       aChild_    := GetWindowRect(hWnd)
       iCWidth    := aChild_[3] - aChild_[1]
       iCHeight   := aChild_[4] - aChild_[2]
       hWndParent := GetWindow(hWnd,GW_OWNER)
       aParent_   := GetClientRect(hWndParent)
       aPoint_    := {(aParent_[3]/2),(aParent_[4]/2)}
       ClienttoScreen(hWndParent,aPoint_)
       aPoint_[1] -= (iCWidth  / 2)
       aPoint_[2] -= (iCHeight / 2)
       ScreentoClient(hWndParent,aPoint_)
       aPoint_[1] := max(0, aPoint_[1])
       aPoint_[2] := max(0, aPoint_[2])
       ClienttoScreen(hWndParent,aPoint_)
       MoveWindow(hWnd,aPoint_[1],aPoint_[2],iCWidth,iCHeight,.F.)
RETURN(NIL)

FUNCTION GetWindowRect(hWnd)
       LOCAL cGWR
       LOCAL cBuf := space(8)
       LOCAL hLib := LoadLibrary("USER.EXE")

       cGWR := GetProcAddress(hLib, "GetWindowRect", ;
                              "Pascal", "void", "HWND, string")
       CallDLL(cGWR, hWnd, @cBuf)
       FreeLibrary(hLib)
RETURN(bin2a(cBuf, "int[4]"))

// These two functions are needed to handle menus in resource files.
// WEnableMenuItem() replaces the EnableMenuItem() function in the Clip-4-Win
// lib.
//
FUNCTION WEnableMenuItem(hWnd,nItem,nFlags)
       LOCAL cENM
       LOCAL hLib := LoadLibrary("USER.EXE")

       cENM := GetProcAddress(hLib, "EnableMenuItem", ;
                              "Pascal", "void", "HWND, int,int")
       CallDLL(cENM, hWnd, nItem, nFlags)
       FreeLibrary(hLib)
RETURN(NIL)

FUNCTION TextHeight
         LOCAL aTM := {}
         LOCAL hDC := GetDC(hMainWnd)

         GetTextMetrics(hDC,@aTM)
         ReleaseDC(hMainWnd,hDC)
RETURN(aTM[ TM_Height ])

// LTOC() - Converts a logical .T. or .F. to a character string
// Mainly used for debugging...
//
FUNCTION LTOC(lVal)
RETURN(IIF(lVal,'.T.','.F.'))

// LdBrowFlds() - Returns an array for use by WBrowse() that contains
//                 ALL fields in a database
FUNCTION LdBrowFlds
       LOCAL nCtr
       LOCAL aFldList := {}
       LOCAL aStruct  := DBSTRUCT()
       LOCAL cPict

       FOR nCtr := 1 TO LEN( aStruct )
           SpinBall()
           cPict := ""
           DO CASE
              CASE aStruct[nCtr][2] == 'C'
                   AADD( aFldList, {aStruct[nCtr][1],"{||"+ALIAS()+"->"+aStruct[nCtr][1]+'}',cPict} )
              CASE aStruct[nCtr][2] == 'N'
                   cPict := REPLICATE( '9', aStruct[nCtr][3] )
                   IF aStruct[nCtr][4] > 0
                      cPict += '.'+REPLICATE( '9', aStruct[nCtr][4] )
                   ENDIF
                   AADD( aFldList, {aStruct[nCtr][1],"{||"+ALIAS()+"->"+aStruct[nCtr][1]+"}",cPict} )
              CASE aStruct[nCtr][2] == 'L'
                   AADD( aFldList, {aStruct[nCtr][1],"{||IIF("+ALIAS()+"->"+aStruct[nCtr][1]+",'.T.','.F.')}",cPict} )
              CASE aStruct[nCtr][2] == 'D'
                   AADD( aFldList, {aStruct[nCtr][1],"{||DTOC(FIELD->"+aStruct[nCtr][1]+")}",cPict} )
              CASE aStruct[nCtr][2] == 'M'
                   AADD( aFldList, {"<MEMO>","{||"+ALIAS()+"->"+aStruct[nCtr][1]+'}',cPict} )
           ENDCASE
           SpinBall()
         NEXT
RETURN(aFldList)
